@@ -7,7 +7,7 @@ from django.db import transaction, connection
 from django.db.models import PositiveIntegerField, BigIntegerField, UniqueConstraint, Q
 from django.forms import model_to_dict
 
-from isc_common import setAttr, delAttr
+from isc_common import setAttr, delAttr, get_list_nums_key_dict
 from isc_common.bit import TurnBitOn
 from isc_common.common.functions import delete_dbl_spaces
 from isc_common.common.mat_views import refresh_mat_view
@@ -15,13 +15,12 @@ from isc_common.fields.related import ForeignKeyProtect
 from isc_common.http.DSRequest import DSRequest
 from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
 from isc_common.models.audit import AuditModel
-from isc_common.progress import managed_progress, ProgressDroped
+from isc_common.progress import managed_progress, ProgressDroped, progress_deleted
 from isc_common.string import get_NoneStr
 from kaf_pas.ckk.models import get_operations_from_trunsaction
 from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
 from kaf_pas.kd.models.documents import Documents
 from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
-from kaf_pas.production.models import progress_deleted
 
 logger = logging.getLogger(__name__)
 
@@ -77,6 +76,26 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         data.update(res)
         return data
 
+    @staticmethod
+    def find_item(item):
+        item_query = None
+        if isinstance(item, int):
+            item = Item.objects.get(id=item)
+        elif not isinstance(item, Item):
+            raise Exception(f'item must be Item or int')
+
+        STMP_2 = item.STMP_2.value_str if item.STMP_2 else None
+        STMP_1 = item.STMP_1.value_str if item.STMP_1 else None
+
+        if STMP_2 == None:
+            item_query = Item.objects.filter(STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1))
+        elif STMP_1 == None:
+            item_query = Item.objects.filter(STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2))
+        else:
+            item_query = Item.objects.filter(STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1), STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2))
+
+        return [item for item in item_query]
+
     @staticmethod
     def check_4_production(item):
         if isinstance(item, int):
@@ -98,7 +117,6 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
 
     @staticmethod
     def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None, props=TurnBitOn(0, 0), document=None, show_progress=True):
-        from isc_common.models.deleted_progresses import Deleted_progresses
         from kaf_pas.ckk.models.item_document import Item_document
         from kaf_pas.ckk.models.item_image_refs import Item_image_refs
         from kaf_pas.ckk.models.item_line import Item_line
@@ -106,11 +124,12 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         from kaf_pas.ckk.models.tmp_item_refs import Tmp_Item_refsManager
         from kaf_pas.production.models.operations_item import Operations_item
 
-        settings.LOCKS.acquire('ItemManager.delete_recursive')
+        key = 'ItemManager.delete_recursive'
+        settings.LOCKS.acquire(key)
         item_lotsman_document_cnt = 0
         if document != None:
             if ItemManager.check_4_production(item=item_id) == True:
-                settings.LOCKS.release('ItemManager.delete_recursive')
+                settings.LOCKS.release(key)
                 return 2
 
         sql_text = None
@@ -158,7 +177,6 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         def _delete(progress=None):
             try:
                 with transaction.atomic():
-                    Tmp_Item_refsManager.create()
                     try:
                         item = Item.objects.get(id=item_id)
                         if cnt == 0:
@@ -176,10 +194,10 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                                 if not soft_delete:
                                     if document != None:
                                         if ItemManager.check_4_production(item=item.child_id) == True:
-                                            _continue=True
+                                            _continue = True
 
                                     if _continue:
-                                        _continue=False
+                                        _continue = False
                                         continue
 
                                     Item_image_refs.objects.filter(item_id=item.child_id).delete()
@@ -197,33 +215,29 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                                     Item_document.objects.filter(item_id=item.child_id).delete()
                                     Item.objects.filter(id=item.child_id).delete()
                                 else:
-                                    Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
-
-                                    if delete_lines:
-                                        Item_line.objects.filter(parent_id=item.child_id).soft_delete()
-                                        Item_line.objects.filter(child_id=item.child_id).soft_delete()
-
-                                        Item_line.objects.filter(parent_id=item.parent_id).soft_delete()
-                                        Item_line.objects.filter(child_id=item.parent_id).soft_delete()
-
-                                    Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
-                                    Item_refs.objects.filter(child_id=item.child_id).soft_delete()
+                                    # Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
+                                    #
+                                    # if delete_lines:
+                                    #     Item_line.objects.filter(parent_id=item.child_id).soft_delete()
+                                    #     Item_line.objects.filter(child_id=item.child_id).soft_delete()
+                                    #
+                                    #     Item_line.objects.filter(parent_id=item.parent_id).soft_delete()
+                                    #     Item_line.objects.filter(child_id=item.parent_id).soft_delete()
+                                    #
+                                    # Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
+                                    # Item_refs.objects.filter(child_id=item.child_id).soft_delete()
                                     Item.objects.filter(id=item.child_id).soft_delete()
 
                                 if progress != None:
-                                    if  progress.step() != 0:
+                                    if progress.step() != 0:
+                                        settings.LOCKS.release(key)
                                         raise ProgressDroped(progress_deleted)
                     except Item.DoesNotExist:
-                        settings.LOCKS.release('ItemManager.delete_recursive')
+                        settings.LOCKS.release(key)
 
             except db.models.deletion.ProtectedError as ex:
                 logger.error(f'Item_refs: {item}')
-                settings.LOCKS.release('ItemManager.delete_recursive')
-                raise ex
-
-            except ProgressDroped as ex:
-                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
-                settings.LOCKS.release('ItemManager.delete_recursive')
+                settings.LOCKS.release(key)
                 raise ex
 
         if show_progress and cnt > 0:
@@ -235,15 +249,73 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                     title='Выполнено',
                     props=props
             ) as progress:
-               _delete(progress)
+                _delete(progress)
         else:
             _delete()
 
-        settings.LOCKS.release('ItemManager.delete_recursive')
+        settings.LOCKS.release(key)
 
         if item_lotsman_document_cnt > 0:
             refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
 
+    @staticmethod
+    def rec(data):
+        _data = data.copy()
+        relevant = data.get('relevant')
+        confirmed = data.get('confirmed')
+        props = data.get('props')
+
+        if relevant == 'Актуален':
+            props |= Item.props.relevant
+        else:
+            props &= ~Item.props.relevant
+
+        if confirmed == 'Подтвержден':
+            props |= Item.props.confirmed
+        else:
+            props &= ~Item.props.confirmed
+
+        where_from = data.get('where_from')
+        if where_from == 'Получено из чертежа':
+            props |= Item.props.from_cdw
+        elif where_from == 'Получено из спецификации':
+            props |= Item.props.from_spw
+        elif where_from == 'Получено из бумажного архива':
+            props |= Item.props.from_pdf
+        elif where_from == 'Занесено вручную':
+            props |= Item.props.man_input
+
+        delAttr(data, 'relevant')
+        delAttr(data, 'confirmed')
+        delAttr(data, 'where_from')
+        delAttr(data, 'qty_operations')
+        delAttr(data, 'refs_props')
+        delAttr(data, 'icon')
+        delAttr(data, 'id')
+        delAttr(data, 'STMP_1__value_str')
+        delAttr(data, 'STMP_2__value_str')
+        delAttr(data, 'document__file_document')
+        delAttr(data, 'isFolder')
+        delAttr(data, 'isLotsman')
+        delAttr(data, 'parent_id')
+        delAttr(data, 'section')
+        delAttr(data, 'used')
+
+        setAttr(data, 'props', props)
+
+        version = ItemManager.get_verstion(
+            STMP_1=data.get('STMP_1_id'),
+            STMP_2=data.get('STMP_2_id'),
+            props=data.get('props')
+        )
+
+        if version != data.get('version'):
+            setAttr(data, 'version', version)
+
+        res = Item.objects.filter(id=_data.get('id')).update(**data)
+
+        return _data
+
     def updateFromRequest(self, request, removed=None, function=None):
         from kaf_pas.ckk.models.item_refs import Item_refs
         from kaf_pas.ckk.models.item_line import Item_line
@@ -317,64 +389,10 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                 return res
 
             else:
-                def rec(data):
-                    _data = data.copy()
-                    relevant = data.get('relevant')
-                    confirmed = data.get('confirmed')
-                    props = data.get('props')
-
-                    if relevant == 'Актуален':
-                        props |= Item.props.relevant
-                    else:
-                        props &= ~Item.props.relevant
-
-                    if confirmed == 'Подтвержден':
-                        props |= Item.props.confirmed
-                    else:
-                        props &= ~Item.props.confirmed
-
-                    where_from = data.get('where_from')
-                    if where_from == 'Получено из чертежа':
-                        props |= Item.props.from_cdw
-                    elif where_from == 'Получено из спецификации':
-                        props |= Item.props.from_spw
-                    elif where_from == 'Получено из бумажного архива':
-                        props |= Item.props.from_pdf
-                    elif where_from == 'Занесено вручную':
-                        props |= Item.props.man_input
-
-                    delAttr(data, 'relevant')
-                    delAttr(data, 'confirmed')
-                    delAttr(data, 'where_from')
-                    delAttr(data, 'qty_operations')
-                    delAttr(data, 'refs_props')
-                    delAttr(data, 'icon')
-                    delAttr(data, 'id')
-                    delAttr(data, 'STMP_1__value_str')
-                    delAttr(data, 'STMP_2__value_str')
-                    delAttr(data, 'document__file_document')
-                    delAttr(data, 'isFolder')
-                    delAttr(data, 'isLotsman')
-                    delAttr(data, 'parent_id')
-                    delAttr(data, 'section')
-
-                    setAttr(data, 'props', props)
-
-                    res = Item.objects.filter(id=_data.get('id')).update(**data)
-
-                    return _data
-
-                if isinstance(data, dict):
-                    for k, _data in data.items():
-                        if isinstance(_data, dict):
-                            res = rec(_data)
-                        else:
-                            res = rec(data.copy())
-                            break
-                else:
-                    res = 0
-
-            return res
+                res = []
+                for _data in get_list_nums_key_dict(data):
+                    res.append(ItemManager.rec(_data.copy()))
+                return res
 
     def deleteFromRequest(self, request, removed=None, ):
         from kaf_pas.ckk.models.item_refs import Item_refs
@@ -385,11 +403,12 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         res = 0
 
         mode = None
+        key = 'ItemManager.deleteFromRequest'
         with transaction.atomic():
             operations = get_operations_from_trunsaction(data)
             if isinstance(operations, list):
                 for operation in get_operations_from_trunsaction(data):
-                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
+                    settings.LOCKS.acquire(key)
                     _data = operation.get('data')
                     if _data.get('mode') == 'deleteInner':
                         mode = 'deleteInner'
@@ -398,26 +417,29 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                         if isinstance(records, list):
                             for record in records:
                                 res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
-                    settings.LOCKS.release('ItemManager.deleteFromRequest')
+                    settings.LOCKS.release(key)
 
                 if mode == 'deleteInner':
                     return res
             elif isinstance(data, dict):
                 if data.get('mode') == 'deleteInner':
-                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
+                    settings.LOCKS.acquire(key)
                     records = data.get('records')
 
                     if isinstance(records, list):
                         for record in records:
-                            res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
-                        settings.LOCKS.release('ItemManager.deleteFromRequest')
+                            if record.get('parent_id') == None:
+                                res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent__isnull=True).delete_with_checked()
+                            else:
+                                res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
+                        settings.LOCKS.release(key)
                         return res
-                    settings.LOCKS.release('ItemManager.deleteFromRequest')
+                    settings.LOCKS.release(key)
                 elif data.get('mode') == 'reloadRefs':
                     records = data.get('records')
                     res = 0
                     if isinstance(records, list):
-                        settings.LOCKS.acquire('ItemManager.deleteFromRequest')
+                        settings.LOCKS.acquire(key)
                         for record in records:
                             id = record.get('id')
                             parent_id = record.get('parent_id')
@@ -425,8 +447,9 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                             for item_line in Item_line.objects.filter(parent_id=id):
                                 item_refs, _ = Item_refs.objects.get_or_create(parent_id=id, child=item_line.child)
                                 res += 1
-                        settings.LOCKS.release('ItemManager.deleteFromRequest')
+                        settings.LOCKS.release(key)
 
+                    settings.LOCKS.release(key)
                     return res
 
         for id, mode in request.get_tuple_ids():
@@ -457,18 +480,40 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         return res
 
     @staticmethod
-    def get_verstion(STMP_1, STMP_2, props, inc=1):
-        settings.LOCKS.acquire('ItemManager.get_verstion')
-        version = None
+    def get_verstion(STMP_1, STMP_2, props, version=None, inc=1):
+        key = 'ItemManager.get_verstion'
+        settings.LOCKS.acquire(key)
         while True:
-            res = Item.objects.filter(
-                STMP_1=STMP_1,
-                STMP_2=STMP_2,
-                props=props,
-                version=version
-            )
+            if isinstance(STMP_1, int):
+                STMP_1 = Document_attributes.objects.get(id=STMP_1)
+
+            if isinstance(STMP_2, int):
+                STMP_2 = Document_attributes.objects.get(id=STMP_2)
+
+            if STMP_1 == None:
+                res = Item.objects.filter(
+                    STMP_1__isnull=True,
+                    STMP_2=STMP_2,
+                    props=props,
+                    version=version
+                )
+            elif STMP_2 == None:
+                res = Item.objects.filter(
+                    STMP_1=STMP_1,
+                    STMP_2__isnull=True,
+                    props=props,
+                    version=version
+                )
+            else:
+                res = Item.objects.filter(
+                    STMP_1=STMP_1,
+                    STMP_2=STMP_2,
+                    props=props,
+                    version=version
+                )
+
             if len(res) == 0:
-                settings.LOCKS.release('ItemManager.get_verstion')
+                settings.LOCKS.release(key)
                 return version
             else:
                 version = res[0].version
@@ -596,6 +641,39 @@ class Item(AuditModel):
 
     objects = ItemManager()
 
+    @property
+    def relevant(self):
+        if self.props.relevant:
+            return 'Актуален'
+        else:
+            return 'Актуален'
+
+    @property
+    def confirmed(self):
+        if self.props.confirmed:
+            return 'Подтвержден'
+        else:
+            return 'Не подтвержден'
+
+    @property
+    def where_from(self):
+        if self.props.from_cdw:
+            return 'Получено из чертежа'
+        elif self.props.from_spw:
+            return 'Получено из спецификации'
+        elif self.props.for_line:
+            return 'Строка спецификации'
+        elif self.props.from_pdf:
+            return 'Получено из бумажного архива'
+        elif self.props.man_input:
+            return 'Занесено вручную'
+        elif self.props.copied:
+            return 'Скопировано'
+        elif self.props.from_lotsman:
+            return 'Получено из Лоцмана'
+        else:
+            return ''
+
     @property
     def item_name(self):
         if self.STMP_1 != None and self.STMP_2 != None:
