@@ -7,6 +7,7 @@ from django.forms import model_to_dict
 
 from isc_common import setAttr, delAttr
 from isc_common.bit import TurnBitOn
+from isc_common.common.functions import delete_dbl_spaces
 from isc_common.fields.code_field import CodeField
 from isc_common.fields.name_field import NameField
 from isc_common.fields.related import ForeignKeyProtect
@@ -15,6 +16,7 @@ from isc_common.models.audit import AuditModel, AuditQuerySet
 from isc_common.models.tree_audit import TreeAuditModelManager
 from isc_common.number import StrToNumber
 from isc_common.progress import managed_progress, ProgressDroped
+from kaf_pas.ckk.models.ed_izm import Ed_izm
 from kaf_pas.ckk.models.item import Item
 from kaf_pas.ckk.models.item_refs import Item_refs
 from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
@@ -61,6 +63,7 @@ class Item_lineQuerySet(AuditQuerySet):
         section_num = Item_lineManager.section_num(section)
         setAttr(kwargs, 'section_num', section_num)
         delAttr(kwargs, 'confirmed')
+        delAttr(kwargs, 'edizm__name')
         return super().update(**kwargs)
 
     def update_or_create(self, defaults=None, **kwargs):
@@ -117,6 +120,8 @@ class Item_lineManager(TreeAuditModelManager):
             'SPC_CLM_KOD__value_str': record.SPC_CLM_KOD.value_str if record.SPC_CLM_KOD else None,
             'SPC_CLM_FACTORY_id': record.SPC_CLM_FACTORY.id if record.SPC_CLM_FACTORY else None,
             'SPC_CLM_FACTORY__value_str': record.SPC_CLM_FACTORY.value_str if record.SPC_CLM_FACTORY else None,
+            'edizm_id': record.edizm.id if record.edizm else None,
+            'edizm__name': record.edizm.name if record.edizm else None,
             'section': record.section,
             'section_num': record.section_num,
             'subsection': record.subsection,
@@ -250,6 +255,8 @@ class Item_lineManager(TreeAuditModelManager):
                 targetLine.save()
 
                 Item_refs.objects.get_or_create(child=targetLine.child, parent=targetLine.parent)
+                setAttr(targetRecord, 'confirmed', dropRecord.get('confirmed'))
+                setAttr(targetRecord, 'relevant', dropRecord.get('relevant'))
 
                 return targetRecord
 
@@ -260,13 +267,19 @@ class Item_lineManager(TreeAuditModelManager):
 
         if relevant == 'Актуален':
             item_props |= Item.props.relevant
+            # setAttr(data_res, 'relevant', 'Актуален')
+
         elif relevant == 'Не актуален':
             item_props &= ~Item.props.relevant
+            # setAttr(data_res, 'relevant', 'Не актуален')
 
         if confirmed == 'Подтвержден':
             item_props |= Item.props.confirmed
+            # setAttr(data_res, 'confirmed', 'Подтвержден')
+
         elif confirmed == 'Не подтвержден':
             item_props &= ~Item.props.confirmed
+            # setAttr(data_res, 'confirmed', 'Не подтвержден')
 
         delAttr(data, 'document_id')
         delAttr(data, 'isFolder')
@@ -291,11 +304,11 @@ class Item_lineManager(TreeAuditModelManager):
 
     def copyFromRequest(self, request, removed=None, ):
         from kaf_pas.ckk.models.item_line_view import Item_line_view
-        from isc_common.models.deleted_progresses import Deleted_progresses
 
         request = DSRequest(request=request)
 
         data = request.json.get('data')
+        res = None
         with managed_progress(
                 qty=len(data.get('records')),
                 id=str(uuid.uuid4()),
@@ -303,38 +316,57 @@ class Item_lineManager(TreeAuditModelManager):
                 title="Выполнено",
                 props=TurnBitOn(0, 0)
         ) as progress:
-            try:
-                with transaction.atomic():
-                    progress.setContentsLabel(f'<b>Создание строк детализации</b>')
-                    for record in data.get('records'):
-                        for item_line in Item_line_view.objects.filter(
-                                SPC_CLM_NAME__value_str=record.get('STMP_1__value_str'),
-                                SPC_CLM_MARK__value_str=record.get('STMP_2__value_str')
-                        ):
-                            res, _ = Item_line.objects.get_or_create(
-                                parent_id=record.get("parent_id"),
-                                child_id=record.get("id"),
-                                SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
-                                SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
-                                SPC_CLM_POS=item_line.SPC_CLM_POS,
-                                SPC_CLM_MARK=item_line.SPC_CLM_MARK,
-                                SPC_CLM_NAME=item_line.SPC_CLM_NAME,
-                                SPC_CLM_COUNT=item_line.SPC_CLM_COUNT,
-                                SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
-                                SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
-                                SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
-                                SPC_CLM_USER=item_line.SPC_CLM_USER,
-                                SPC_CLM_KOD=item_line.SPC_CLM_USER,
-                                SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
-                                section=item_line.section,
-                                section_num=item_line.section_num,
-                                subsection=item_line.subsection,
-                            )
-                            break
-                        progress.step()
-            except ProgressDroped as ex:
-                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
-                raise ex
+            with transaction.atomic():
+                progress.setContentsLabel(f'<b>Создание строк детализации</b>')
+                for record in data.get('records'):
+                    if record.get("parent_id") == None:
+                        raise Exception('Верхний уровень не подлежит внесению в детализацию.')
+
+                    query = Item_line_view.objects.filter(
+                        SPC_CLM_NAME_id=record.get('STMP_1_id'),
+                        SPC_CLM_MARK_id=record.get('STMP_2_id')
+                    )
+
+                    if query.count() == 0:
+                        query = Item_line_view.objects.filter(
+                            SPC_CLM_NAME__value_str__delete_dbl_spaces=delete_dbl_spaces(record.get('STMP_1__value_str')),
+                            SPC_CLM_MARK__value_str__delete_dbl_spaces=delete_dbl_spaces(record.get('STMP_2__value_str'))
+                        )
+
+                    if query.count() > 0:
+                        item_line = query[0]
+                        res, _ = Item_line.objects.get_or_create(
+                            parent_id=record.get("parent_id"),
+                            child_id=record.get("id"),
+                            SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
+                            SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
+                            SPC_CLM_POS=item_line.SPC_CLM_POS,
+                            SPC_CLM_MARK=item_line.SPC_CLM_MARK,
+                            SPC_CLM_NAME=item_line.SPC_CLM_NAME,
+                            SPC_CLM_COUNT=item_line.SPC_CLM_COUNT,
+                            SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
+                            SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
+                            SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
+                            SPC_CLM_USER=item_line.SPC_CLM_USER,
+                            SPC_CLM_KOD=item_line.SPC_CLM_USER,
+                            SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
+                            section=item_line.section,
+                            section_num=item_line.section_num,
+                            subsection=item_line.subsection,
+                        )
+                    else:
+                        res = Item_line.objects.create(
+                            parent_id=record.get("parent_id"),
+                            child_id=record.get("id"),
+                            SPC_CLM_NAME_id=record.get('STMP_1_id'),
+                            SPC_CLM_MARK_id=record.get('STMP_2_id'),
+                            SPC_CLM_COUNT=Document_attributesManager.get_or_create_attribute('SPC_CLM_COUNT', '1')[0],
+                            section='Сборочные единицы',
+                        )
+
+                    if progress.step() != 0:
+                        raise ProgressDroped()
+            return model_to_dict(res)
 
     def lookupFromRequest(self, request):
         request = DSRequest(request=request)
@@ -376,18 +408,19 @@ class Item_line(AuditModel):
     parent = ForeignKeyProtect(Item, verbose_name='Товарная позиция', related_name='item_parent')
     child = ForeignKeyProtect(Item, verbose_name='Товарная позиция', related_name='item_child')
 
-    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, verbose_name='Формат', related_name='SPC_CLM_FORMAT', null=True, blank=True)
-    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, verbose_name='Зона', related_name='SPC_CLM_ZONE', null=True, blank=True)
-    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, verbose_name='Позиция', related_name='SPC_CLM_POS', null=True, blank=True)
-    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение', related_name='SPC_CLM_MARK', null=True, blank=True)
-    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, verbose_name='Наименование', related_name='SPC_CLM_NAME', null=True, blank=True)
-    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, verbose_name='Количество', related_name='SPC_CLM_COUNT', null=True, blank=True)
-    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, verbose_name='Примечание', related_name='SPC_CLM_NOTE', null=True, blank=True)
-    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, verbose_name='Масса', related_name='SPC_CLM_MASSA', null=True, blank=True)
-    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, verbose_name='Материал', related_name='SPC_CLM_MATERIAL', null=True, blank=True)
-    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, verbose_name='Пользовательская', related_name='SPC_CLM_USER', null=True, blank=True)
-    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, verbose_name='Код', related_name='SPC_CLM_KOD', null=True, blank=True)
-    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, verbose_name='Предприятие - изготовитель', related_name='SPC_CLM_FACTORY', null=True, blank=True)
+    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, verbose_name='Формат', related_name='Item_line_SPC_CLM_FORMAT', null=True, blank=True)
+    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, verbose_name='Зона', related_name='Item_line_SPC_CLM_ZONE', null=True, blank=True)
+    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, verbose_name='Позиция', related_name='Item_line_SPC_CLM_POS', null=True, blank=True)
+    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение', related_name='Item_line_SPC_CLM_MARK', null=True, blank=True)
+    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, verbose_name='Наименование', related_name='Item_line_SPC_CLM_NAME', null=True, blank=True)
+    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, verbose_name='Количество', related_name='Item_line_SPC_CLM_COUNT', null=True, blank=True)
+    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, verbose_name='Примечание', related_name='Item_line_SPC_CLM_NOTE', null=True, blank=True)
+    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, verbose_name='Масса', related_name='Item_line_SPC_CLM_MASSA', null=True, blank=True)
+    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, verbose_name='Материал', related_name='Item_line_SPC_CLM_MATERIAL', null=True, blank=True)
+    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, verbose_name='Пользовательская', related_name='Item_line_SPC_CLM_USER', null=True, blank=True)
+    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, verbose_name='Код', related_name='Item_line_SPC_CLM_KOD', null=True, blank=True)
+    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, verbose_name='Предприятие - изготовитель', related_name='Item_line_SPC_CLM_FACTORY', null=True, blank=True)
+    edizm = ForeignKeyProtect(Ed_izm, verbose_name='Еденица измерения', related_name='Item_line_EdIzm', null=True, blank=True)
     section = CodeField()
     section_num = PositiveIntegerField(null=True, blank=True)
     subsection = NameField()
@@ -423,6 +456,7 @@ class Item_line(AuditModel):
                f'\nSPC_CLM_USER=[{self.SPC_CLM_USER}]' \
                f'\nSPC_CLM_KOD=[{self.SPC_CLM_KOD}' \
                f'\nSPC_CLM_FACTORY=[{self.SPC_CLM_FACTORY}]\n' \
+               f'\nEdIzm=[{self.edizm}]\n' \
                f'section={self.section}\n' \
                f'subsection={self.subsection}\n\n'
 
