@@ -3,7 +3,7 @@ import logging
 from django.db.models import BigIntegerField, BooleanField, PositiveIntegerField
 from django.forms import model_to_dict
 
-from isc_common import setAttr
+from isc_common import setAttr, delAttr
 from isc_common.datetime import DateToStr
 from isc_common.fields.code_field import CodeField
 from isc_common.fields.name_field import NameField
@@ -12,7 +12,7 @@ from isc_common.http.DSRequest import DSRequest
 from isc_common.models.audit import AuditQuerySet, AuditManager
 from isc_common.models.base_ref import Hierarcy
 from isc_common.number import DelProps
-from kaf_pas.ckk.models.item import Item_add
+from kaf_pas.ckk.models.item import Item_add, ItemManager
 from kaf_pas.ckk.models.item_line import Item_lineManager
 from kaf_pas.ckk.models.item_refs import Item_refsManager, Item_refs
 from kaf_pas.kd.models.document_attributes import Document_attributes
@@ -39,6 +39,8 @@ class Item_operations_viewManager(AuditManager):
 
         data = request.get_data()
         refs_id = data.get('refs_id')
+        launch_detail_id = data.get('launch_detail_id')
+        delAttr(data, 'launch_detail_id')
         refs_props = data.get('refs_props')
         if data.get('used') == True:
             refs_props |= Item_refs.props.used
@@ -50,6 +52,9 @@ class Item_operations_viewManager(AuditManager):
 
         Item_refs.objects.update_or_create(id=refs_id, defaults=refs_data)
 
+        ItemManager.rec(data.copy())
+        setAttr(data, 'launch_detail_id', launch_detail_id)
+
         return data
 
     @staticmethod
