@@ -41,7 +41,14 @@ def get_excl():
 
 def Item_view_query():
     # Item_view._meta.db_table = 'ckk_item_mview'
-    query = Item_view.objects.exclude(id__in=get_excl())
+    query = Item_view.objects. \
+        select_related(
+        'STMP_1',
+        'STMP_2',
+        'document',
+        'lotsman_document',
+    ). \
+        exclude(id__in=get_excl())
     return query
 
 
@@ -59,12 +66,34 @@ def Item_view_Fetch(request):
             status=RPCResponseConstant.statusSuccess).response)
 
 
+@JsonResponseWithException()
+def Item_view_4_idFetch(request):
+    _request = DSRequest(request=request)
+    return JsonResponse(
+        DSResponse(
+            request=request,
+            data=Item_view_query().
+                exclude(id__in=[int(item.value) for item in Contants.objects.filter(code__in=['audo_top_level'])]).
+                get_range_rows1(
+                request=request,
+                function=Item_viewManager.getRecord,
+                distinct_field_names=('id',)
+            ),
+            status=RPCResponseConstant.statusSuccess).response)
+
+
 @JsonResponseWithException(printing=False)
 def Item_view_Fetch1(request):
     return JsonResponse(
         DSResponse(
             request=request,
             data=Item_operations_view.objects.
+                select_related(
+                'STMP_1',
+                'STMP_2',
+                'document',
+                # 'lotsman_document',
+            ).
                 exclude(id__in=[int(item.value) for item in Contants.objects.filter(code__in=['audo_top_level'])]).
                 get_range_rows1(request=request, function=Item_operations_viewManager.getRecord, distinct_field_names=('id',)),
             status=RPCResponseConstant.statusSuccess).response)
@@ -72,38 +101,29 @@ def Item_view_Fetch1(request):
 
 @JsonResponseWithException(printing=False)
 def Item_view_Fetch2(request):
-    from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail
-
     _request = DSRequest(request=request)
 
     data = _request.get_data()
 
     with transaction.atomic():
-        ready_2_launch_detail = Ready_2_launch_detail.objects.get(id=data.get('id'))
+        if data.get('launch_detail_id') == None:
+            return JsonResponse(
+                DSResponse(
+                    request=request,
+                    data=Item_view_query().
+                        get_range_rows1(
+                        request=request,
+                        function=Item_viewManager.getRecord,
+                        distinct_field_names=('id',)
+                    ),
+                    status=RPCResponseConstant.statusSuccess).response)
+
+        id = data.get('launch_detail_id')
         tmp_table_name = f'ready_2_launch_detail_tmp_table'
         create_tmp_table(
             on_commit=None,
             drop=False,
-            sql_str=f'''SELECT
-                                       t."isFolder",
-                                       t."STMP_1_id",
-                                       t."STMP_2_id",
-                                       t.deliting,
-                                       t.document__file_document,
-                                       t.document_id,
-                                       t.editing,
-                                       t.id,
-                                       t.lastmodified,
-                                       t.parent_id,
-                                       t.props,
-                                       t.qty_operations,
-                                       t.refs_id,
-                                       t.refs_props,
-                                       t.relevant,
-                                       t.confirmed,
-                                       t.section,
-                                       t.version,
-                                       t.where_from
+            sql_str=f'''SELECT  t.*
                                 FROM production_ready_2_launch_detail s
                                          CROSS JOIN LATERAL
                                     json_to_recordset(s.item_full_name_obj::json) as t(
@@ -128,7 +148,7 @@ def Item_view_Fetch2(request):
                                                                                        "where_from" text
                                                                                         )
                                     where s.id = %s''',
-            params=[ready_2_launch_detail.id],
+            params=[id],
             table_name=tmp_table_name)
 
         return JsonResponse(
@@ -224,16 +244,17 @@ def Item_view_Fetch3(request):
     _request = DSRequest(request=request)
     data = _request.get_data()
 
-    if settings.LOCKS.locked(f'Item_view_Fetch3_{data.get("id")}'):
+    key = f'Item_view_Fetch3_{data.get("id")}'
+    if settings.LOCKS.locked(key):
         redirect("http://ya.ru")
-    settings.LOCKS.acquire(f'Item_view_Fetch3_{data.get("id")}')
+    settings.LOCKS.acquire(key)
     with transaction.atomic():
         Tmp_Item_refsManager.create()
         Tmp_Item_operations_viewManager.create()
 
         UploadTmpTable(data).make_tmp()
 
-        settings.LOCKS.release(f'Item_view_Fetch3_{data.get("id")}')
+        settings.LOCKS.release(key)
         return JsonResponse(
             DSResponse(
                 request=request,
@@ -252,16 +273,17 @@ def Item_view_Fetch4(request):
     _request = DSRequest(request=request)
     data = _request.get_data()
 
-    if settings.LOCKS.locked(f'Item_view_Fetch4_{data.get("id")}'):
+    key = f'Item_view_Fetch4_{data.get("id")}'
+    if settings.LOCKS.locked(key):
         redirect("http://ya.ru")
-    settings.LOCKS.acquire(f'Item_view_Fetch4_{data.get("id")}')
+    settings.LOCKS.acquire(key)
     with transaction.atomic():
         Tmp_Item_refsManager.create()
         Tmp_Item_operations_viewManager.create()
 
         UploadDocumentTmpTable(data).make_tmp()
 
-        settings.LOCKS.release(f'Item_view_Fetch4_{data.get("id")}')
+        settings.LOCKS.release(key)
         return JsonResponse(
             DSResponse(
                 request=request,
@@ -272,6 +294,58 @@ def Item_view_Fetch4(request):
                 status=RPCResponseConstant.statusSuccess).response)
 
 
+@JsonResponseWithException(printing=False)
+def Item_view_Fetch5(request):
+    _request = DSRequest(request=request)
+
+    data = _request.get_data()
+
+    with transaction.atomic():
+        id = data.get('id')
+        tmp_table_name = f'production_order_tmp_table'
+        create_tmp_table(
+            on_commit=None,
+            drop=False,
+            sql_str=f'''SELECT t.*
+                        FROM planing_operations po
+                                 join planing_operation_item poi on po.id = poi.operation_id
+                                 join planing_operation_item_add paoi on poi.item_id = paoi.item_id
+                                 CROSS JOIN LATERAL
+                            json_to_recordset(paoi.item_full_name_obj::json) as t(
+                                                                                 "confirmed" text,
+                                                                                 "deliting" boolean,
+                                                                                 "document__file_document" text,
+                                                                                 "document_id" bigint,
+                                                                                 "editing" boolean,
+                                                                                 "id" bigint,
+                                                                                 "isFolder" boolean,
+                                                                                 "lastmodified" text,
+                                                                                 "parent_id" bigint,
+                                                                                 "props" bigint,
+                                                                                 "qty_operations" int4,
+                                                                                 "refs_id" bigint,
+                                                                                 "refs_props" bigint,
+                                                                                 "relevant" text,
+                                                                                 "section" text,
+                                                                                 "STMP_1_id" bigint,
+                                                                                 "STMP_2_id" bigint,
+                                                                                 "version" int4,
+                                                                                 "where_from" text
+                            )
+                        where po.id = %s''',
+            params=[id],
+            table_name=tmp_table_name)
+
+        return JsonResponse(
+            DSResponse(
+                request=request,
+                data=Item_operations_view.objects.raw(
+                    raw_query=f'select * from {tmp_table_name}',
+                    function=Item_operations_viewManager.getRecord
+                ),
+                status=RPCResponseConstant.statusSuccess).response)
+
+
 @JsonResponseWithException()
 def Item_view_Add(request):
     return JsonResponse(DSResponseAdd(data=Item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)
