@@ -1,10 +1,10 @@
 import logging
 import os
 
-from django.db import transaction, IntegrityError
+from django.db import transaction
+
 from isc_common import setAttr
 from isc_common.common.functions import delete_dbl_spaces
-
 from kaf_pas.kd.models.document_attributes_view import Document_attributes_view
 
 logger = logging.getLogger(__name__)
@@ -269,9 +269,9 @@ class DocumentManagerExt:
         from kaf_pas.ckk.models.item import Item
         from kaf_pas.ckk.models.item import ItemManager
         from kaf_pas.ckk.models.item_refs import Item_refs
-        from kaf_pas.kd.models.document_attributes import Document_attributes
         from kaf_pas.kd.models.documents import Documents
         from kaf_pas.system.models.contants import Contants
+        from kaf_pas.kd.models.document_attributes import Document_attributesManager
 
         top_level = Contants.objects.get(code='audo_top_level')
 
@@ -288,16 +288,9 @@ class DocumentManagerExt:
                     STMP_1 = file_name.strip().replace('.pdf', '').replace('.PDF', '')
                     STMP_2 = STMP_1
 
-                try:
-                    STMP_1_attr, _ = Document_attributes.objects.get_or_create(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1), attr_type=STMP_1_type)
-                except Document_attributes.MultipleObjectsReturned:
-                    STMP_1_attr = Document_attributes.objects.filter(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1), attr_type=STMP_1_type)[0]
-
-                try:
-                    STMP_2_attr, _ = Document_attributes.objects.get_or_create(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2), attr_type=STMP_2_type)
-                except Document_attributes.MultipleObjectsReturned:
-                    STMP_2_attr = Document_attributes.objects.filter(value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2), attr_type=STMP_2_type)[0]
 
+                STMP_1_attr, _ = Document_attributesManager.get_or_create_attribute(attr_codes='STMP_1', value_str=STMP_1)
+                STMP_2_attr, _ = Document_attributesManager.get_or_create_attribute(attr_codes='STMP_2', value_str=STMP_2)
 
                 props = Item.props.relevant | Item.props.from_pdf
                 version = ItemManager.get_verstion(STMP_1=STMP_1_attr, STMP_2=STMP_2_attr, props=props)
@@ -310,6 +303,7 @@ class DocumentManagerExt:
                     version=version
                 )
 
+                logger.debug(f'item: {item.item_name}')
                 Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                 self.link_image_to_item(item)
 
