@@ -17,7 +17,8 @@ logger = logging.getLogger(__name__)
 class Documents_thumbManager(CryptoManager):
     @staticmethod
     def refreshFromDbls(doc_type, table_name):
-        settings.LOCKS.acquire('Documents_thumbManager.refreshFromDbls')
+        key = 'Documents_thumbManager.refreshFromDbls'
+        settings.LOCKS.acquire(key)
         with connection.cursor() as cursor:
             cursor.execute(f'''select
                                     count (*),
@@ -56,7 +57,7 @@ class Documents_thumbManager(CryptoManager):
                         print(f'Deleted: {id}')
                     else:
                         first_step = False
-        settings.LOCKS.release('Documents_thumbManager.refreshFromDbls')
+        settings.LOCKS.release(key)
 
     @staticmethod
     def getRecord(record):
