@@ -6,8 +6,7 @@ from isc_common import Stack
 from isc_common.bit import TurnBitOn
 from isc_common.common.mat_views import create_tmp_mat_view, drop_mat_view
 from isc_common.logger.Logger import Logger
-from isc_common.progress import managed_progress, ProgressDroped
-from kaf_pas.production.models import progress_deleted
+from isc_common.progress import managed_progress, ProgressDroped, progress_deleted
 
 logger = logging.getLogger(__name__)
 logger.__class__ = Logger
@@ -122,161 +121,156 @@ class Lotsman_documents_hierarcyManagerExt:
                         title='Выполнено',
                         props=TurnBitOn(0, 0)
                 ) as progress:
-                    try:
-                        with transaction.atomic():
+                    with transaction.atomic():
 
-                            Tmp_Item_refsManager.create()
-                            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name}  where props=1 order by level'''):
-                                if lotsman_documents_hierarcy.attr_name not in ['Материал']:
+                        Tmp_Item_refsManager.create()
+                        for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name}  where props=1 order by level'''):
+                            if lotsman_documents_hierarcy.attr_name not in ['Материал']:
 
-                                    item, created1 = self.get_item(
-                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
+                                item, created1 = self.get_item(
+                                    lotsman_documents_hierarcy=lotsman_documents_hierarcy,
+                                    items_pairs=items_pairs,
+                                )
+
+                                if lotsman_documents_hierarcy.parent_id != None:
+                                    try:
+                                        _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.get(id=lotsman_documents_hierarcy.parent_id)
+                                    except Lotsman_documents_hierarcy_view.DoesNotExist as ex:
+                                        raise ex
+                                    except Lotsman_documents_hierarcy_view.MultipleObjectsReturned:
+                                        _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.filter(id=lotsman_documents_hierarcy.parent_id).distinct('id')[0]
+
+                                    parent, created2 = self.get_item(
+                                        lotsman_documents_hierarcy=_lotsman_documents_hierarcy,
                                         items_pairs=items_pairs,
                                     )
 
-                                    if lotsman_documents_hierarcy.parent_id != None:
-                                        try:
-                                            _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.get(id=lotsman_documents_hierarcy.parent_id)
-                                        except Lotsman_documents_hierarcy_view.DoesNotExist as ex:
-                                            raise ex
-                                        except Lotsman_documents_hierarcy_view.MultipleObjectsReturned:
-                                            _lotsman_documents_hierarcy = Lotsman_documents_hierarcy_view.objects.filter(id=lotsman_documents_hierarcy.parent_id).distinct('id')[0]
+                                    item_refs, created2 = Item_refs.objects.get_or_create(parent=parent, child=item)
 
-                                        parent, created2 = self.get_item(
-                                            lotsman_documents_hierarcy=_lotsman_documents_hierarcy,
-                                            items_pairs=items_pairs,
-                                        )
+                                # FOR DEBUG
+                                # if created == True or created1 == True or created2 == True:
+                                #     pass
+                                # END FOR DEBUG
 
-                                        item_refs, created2 = Item_refs.objects.get_or_create(parent=parent, child=item)
+                            else:
+                                Lotsman_documents_hierarcy.objects.update_or_create(
+                                    id=lotsman_documents_hierarcy.id,
+                                    defaults=dict(
+                                        props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
+                                    ))
 
-                                    # FOR DEBUG
-                                    # if created == True or created1 == True or created2 == True:
-                                    #     pass
-                                    # END FOR DEBUG
+                            res = progress.step()
+                            if res != 0:
+                                raise ProgressDroped(progress_deleted)
 
-                                else:
-                                    Lotsman_documents_hierarcy.objects.update_or_create(
-                                        id=lotsman_documents_hierarcy.id,
-                                        defaults=dict(
-                                            props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
-                                        ))
+                        progress.close()
+                        with connection.cursor() as cursor:
+                            cursor.execute(f'''select count(*) from {mat_view_name} where props=1 and attr_name=%s''', ['''Сборочная единица'''])
+                            assmbl_progress_count, = cursor.fetchone()
 
-                                res = progress.step()
+                        with managed_progress(
+                                id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
+                                qty=assmbl_progress_count,
+                                user=self.user,
+                                message=f'''Создание сборочных единиц. ({assmbl_progress_count}  шт.)''',
+                                title='Выполнено',
+                                props=TurnBitOn(0, 0)
+                        ) as assmbl_progress:
+
+                            # logger.logging(f'''Сборочные единицы SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
+                            # logger.logging(f'qty: {assmbl_progress_count}', 'debug')
+
+                            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Сборочная единица''']):
+
+                                assmbl_progress.setContentsLabel(f'''<h3>Создание сборочной единицы. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
+                                                                                                      {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
+                                item, _ = self.get_item(
+                                    lotsman_documents_hierarcy=lotsman_documents_hierarcy,
+                                    items_pairs=items_pairs,
+                                )
+
+                                self.make_lines(
+                                    parent=item,
+                                    items_pairs=items_pairs,
+                                    mat_view_name=mat_view_name,
+                                    user_id=self.user,
+                                    record=lotsman_documents_hierarcy_view_record,
+                                )
+
+                                Lotsman_documents_hierarcy.objects.update_or_create(
+                                    id=lotsman_documents_hierarcy.id,
+                                    defaults=dict(
+                                        props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
+                                    ))
+
+                                res = assmbl_progress.step()
                                 if res != 0:
                                     raise ProgressDroped(progress_deleted)
 
-                            progress.close()
-                            with connection.cursor() as cursor:
-                                cursor.execute(f'''select count(*) from {mat_view_name} where props=1 and attr_name=%s''', ['''Сборочная единица'''])
-                                assmbl_progress_count, = cursor.fetchone()
+                        with connection.cursor() as cursor:
+                            cursor.execute(f'''select count(*) from {mat_view_name} where props=1 and attr_name=%s''', ['''Чертеж'''])
+                            assmbl_progress_count, = cursor.fetchone()
 
-                            with managed_progress(
-                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
-                                    qty=assmbl_progress_count,
-                                    user=self.user,
-                                    message=f'''Создание сборочных единиц. ({assmbl_progress_count}  шт.)''',
-                                    title='Выполнено',
-                                    props=TurnBitOn(0, 0)
-                            ) as assmbl_progress:
+                        with managed_progress(
+                                id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
+                                qty=assmbl_progress_count,
+                                user=self.user,
+                                message=f'''Привязка чертежей. ({assmbl_progress_count}  шт.)''',
+                                title='Выполнено',
+                                props=TurnBitOn(0, 0)
+                        ) as assmbl_progress:
 
-                                # logger.logging(f'''Сборочные единицы SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
-                                # logger.logging(f'qty: {assmbl_progress_count}', 'debug')
+                            # logger.logging(f'''Чертежи SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
+                            # logger.logging(f'qty: {assmbl_progress_count}', 'debug')
 
-                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Сборочная единица''']):
+                            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Чертеж''']):
 
-                                    assmbl_progress.setContentsLabel(f'''<h3>Создание сборочной единицы. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
-                                                                                                          {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
-                                    item, _ = self.get_item(
-                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
-                                        items_pairs=items_pairs,
-                                    )
+                                assmbl_progress.setContentsLabel(f'''<h3>Привязка чертежей. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
+                                                                                                {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
+                                item, _ = self.get_item(
+                                    lotsman_documents_hierarcy=lotsman_documents_hierarcy,
+                                    items_pairs=items_pairs,
+                                )
 
-                                    self.make_lines(
-                                        parent=item,
-                                        items_pairs=items_pairs,
-                                        mat_view_name=mat_view_name,
-                                        user_id=self.user,
-                                        record=lotsman_documents_hierarcy_view_record,
-                                    )
+                                for item_ref in Item_refs.objects.filter(child=item):
+                                    if item_ref.parent.lotsman_document.attr_type.code != 'Сборочная единица':
+                                        self.documentManagerExt.link_image_to_lotsman_item(
+                                            lotsman_document=lotsman_documents_hierarcy,
+                                            item=item_ref.parent,
+                                        )
+                                        item_ref.delete()
 
-                                    Lotsman_documents_hierarcy.objects.update_or_create(
-                                        id=lotsman_documents_hierarcy.id,
-                                        defaults=dict(
-                                            props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
-                                        ))
+                                Lotsman_documents_hierarcy.objects.update_or_create(
+                                    id=lotsman_documents_hierarcy.id,
+                                    defaults=dict(
+                                        props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
+                                    ))
 
-                                    res = assmbl_progress.step()
-                                    if res != 0:
-                                        raise ProgressDroped(progress_deleted)
+                                res = assmbl_progress.step()
+                                if res != 0:
+                                    raise ProgressDroped(progress_deleted)
 
-                            with connection.cursor() as cursor:
-                                cursor.execute(f'''select count(*) from {mat_view_name} where props=1 and attr_name=%s''', ['''Чертеж'''])
-                                assmbl_progress_count, = cursor.fetchone()
+                        with managed_progress(
+                                id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
+                                qty=assmbl_progress_count,
+                                user=self.user,
+                                message=f'''Перемещение чертежей. ({assmbl_progress_count}  шт.)''',
+                                title='Выполнено',
+                                props=TurnBitOn(0, 0)
+                        ) as assmbl_progress:
 
-                            with managed_progress(
-                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
-                                    qty=assmbl_progress_count,
-                                    user=self.user,
-                                    message=f'''Привязка чертежей. ({assmbl_progress_count}  шт.)''',
-                                    title='Выполнено',
-                                    props=TurnBitOn(0, 0)
-                            ) as assmbl_progress:
+                            for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Чертеж''']):
+                                item, _ = self.get_item(
+                                    lotsman_documents_hierarcy=lotsman_documents_hierarcy,
+                                    items_pairs=items_pairs,
+                                )
 
-                                # logger.logging(f'''Чертежи SPC_CLM_NAME: {lotsman_documents_hierarcy_view_record.SPC_CLM_NAME.value_str}''', 'debug')
-                                # logger.logging(f'qty: {assmbl_progress_count}', 'debug')
+                                if Item_refs.objects.filter(child=item).count() == 0 and Item_refs.objects.filter(parent=item).count() == 0:
+                                    item.delete()
 
-                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Чертеж''']):
-
-                                    assmbl_progress.setContentsLabel(f'''<h3>Привязка чертежей. ({lotsman_documents_hierarcy.SPC_CLM_NAME.value_str if lotsman_documents_hierarcy.SPC_CLM_NAME else ''} : 
-                                                                                                    {lotsman_documents_hierarcy.SPC_CLM_MARK.value_str if lotsman_documents_hierarcy.SPC_CLM_MARK else ''})</h3>''')
-                                    item, _ = self.get_item(
-                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
-                                        items_pairs=items_pairs,
-                                    )
-
-                                    for item_ref in Item_refs.objects.filter(child=item):
-                                        if item_ref.parent.lotsman_document.attr_type.code != 'Сборочная единица':
-                                            self.documentManagerExt.link_image_to_lotsman_item(
-                                                lotsman_document=lotsman_documents_hierarcy,
-                                                item=item_ref.parent,
-                                            )
-                                            item_ref.delete()
-
-                                    Lotsman_documents_hierarcy.objects.update_or_create(
-                                        id=lotsman_documents_hierarcy.id,
-                                        defaults=dict(
-                                            props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
-                                        ))
-
-                                    res = assmbl_progress.step()
-                                    if res != 0:
-                                        raise ProgressDroped(progress_deleted)
-
-                            with managed_progress(
-                                    id=f'''assmbl_{lotsman_documents_hierarcy_view_record.id}''',
-                                    qty=assmbl_progress_count,
-                                    user=self.user,
-                                    message=f'''Перемещение чертежей. ({assmbl_progress_count}  шт.)''',
-                                    title='Выполнено',
-                                    props=TurnBitOn(0, 0)
-                            ) as assmbl_progress:
-
-                                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where props=1 and attr_name=%s order by level''', ['''Чертеж''']):
-                                    item, _ = self.get_item(
-                                        lotsman_documents_hierarcy=lotsman_documents_hierarcy,
-                                        items_pairs=items_pairs,
-                                    )
-
-                                    if Item_refs.objects.filter(child=item).count() == 0 and Item_refs.objects.filter(parent=item).count() == 0:
-                                        item.delete()
-
-                                    res = assmbl_progress.step()
-                                    if res != 0:
-                                        raise ProgressDroped(progress_deleted)
-
-                    except ProgressDroped as ex:
-                        Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
-                        raise ex
+                                res = assmbl_progress.step()
+                                if res != 0:
+                                    raise ProgressDroped(progress_deleted)
 
             # FOR DEBUG
             with connection.cursor() as cursor:
