@@ -1,5 +1,7 @@
 import logging
 
+from bitfield import BitField
+
 from isc_common.auth.models.user import User
 from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
@@ -14,6 +16,12 @@ class Operation_executorQuerySet(AuditQuerySet):
 
 class Operation_executorManager(AuditManager):
 
+    @staticmethod
+    def props():
+        return BitField(flags=(
+            ('relevant', 'Актуальность'),  # 1
+        ), default=1, db_index=True)
+
     @staticmethod
     def getRecord(record):
         res = {
@@ -31,6 +39,7 @@ class Operation_executorManager(AuditManager):
 class Operation_executor(AuditModel):
     operation = ForeignKeyCascade(Operations)
     executor = ForeignKeyProtect(User)
+    props = Operation_executorManager.props()
 
     objects = Operation_executorManager()
 
