@@ -12,7 +12,10 @@ logger = logging.getLogger(__name__)
 
 
 class Operation_refsQuerySet(TreeAuditModelQuerySet):
-    pass
+    def delete(self):
+        self.check_dliting = False
+
+        return super().delete()
 
 
 class Operation_refsManager(TreeAuditModelManager):
@@ -20,9 +23,11 @@ class Operation_refsManager(TreeAuditModelManager):
     @staticmethod
     def props():
         return BitField(flags=(
-            ('inner_routing', 'Связь операций внутри товарной позиции'),  # 1
-            ('outer_routing', 'Связь операций между товарными позициями'),  # 2
-            ('product_order_routing', 'Связи в блоке заказы на производство'),  # 4
+            ('inner_routing', 'Связь операций внутри товарной позиции'),  # 0-1
+            ('outer_routing', 'Связь операций между товарными позициями'),  # 1-2
+            ('product_order_routing', 'Связи в блоке задания на производство'),  # 2-4
+            ('product_making', 'Связи выпуска'),  # 3-8
+            ('product_making_block', 'Блок операций выпуска, удаляется как одно целоеб для правильного удаления релизов'),  # 3-8
         ), default=0, db_index=True)
 
     @staticmethod
@@ -48,14 +53,13 @@ class Operation_refs(AuditModel):
     objects = Operation_refsManager()
 
     def __str__(self):
-        return f"ID:{self.id}, child: [{self.child}] , parent: [{self.parent}]"
+        return f"ID:{self.id},\n child: [{self.child}],\n parent: [{self.parent}]"
 
     def __repr__(self):
         return self.__str__()
 
     class Meta:
         verbose_name = 'Кросс таблица операций планирования'
-        unique_together = ('child', 'parent', 'props')
         constraints = [
             CheckConstraint(check=~Q(child=F('parent')), name='Operation_refs_not_circulate_refs'),
             UniqueConstraint(fields=['child'], condition=Q(parent=None) & Q(parent_real=None), name='Operation_refs_unique_constraint_0'),
