@@ -1,7 +1,10 @@
 import logging
 
+from django.db.models import CheckConstraint, UniqueConstraint, F, Q
+
 from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
 from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
+from kaf_pas.ckk.models.locations import Locations
 from kaf_pas.planing.models.operations import Operations
 from kaf_pas.production.models.resource import Resource
 
@@ -9,7 +12,8 @@ logger = logging.getLogger(__name__)
 
 
 class Operation_resourcesQuerySet(AuditQuerySet):
-    pass
+    def create(self, **kwargs):
+        return super().create(**kwargs)
 
 
 class Operation_resourcesManager(AuditManager):
@@ -30,6 +34,7 @@ class Operation_resourcesManager(AuditManager):
 class Operation_resources(AuditModel):
     operation = ForeignKeyCascade(Operations, related_name='planing_operation_res')
     resource = ForeignKeyProtect(Resource, related_name='planing_resource_res')
+    location_fin = ForeignKeyProtect(Locations, related_name='planing_location_fin_res', null=True, blank=True)
 
     objects = Operation_resourcesManager()
 
@@ -41,4 +46,7 @@ class Operation_resources(AuditModel):
 
     class Meta:
         verbose_name = 'Кросс таблица'
-        unique_together = (('operation', 'resource'),)
+        constraints = [
+            UniqueConstraint(fields=['operation', 'resource'], condition=Q(location_fin=None), name='Operation_resources_unique_constraint_01'),
+            UniqueConstraint(fields=['location_fin', 'operation', 'resource'], name='Operation_resources_unique_constraint_11'),
+        ]
