@@ -1,7 +1,7 @@
 import logging
 
 from bitfield import BitField
-from django.db.models import DecimalField, FloatField
+from django.db.models import DecimalField
 
 from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
@@ -12,7 +12,10 @@ logger = logging.getLogger(__name__)
 
 
 class Operation_valueQuerySet(AuditQuerySet):
-    pass
+    def create(self, **kwargs):
+        if kwargs.get('value') == 0:
+            raise Exception(f'value must be not 0 !!!')
+        return super().create(**kwargs)
 
 
 class Operation_valueManager(AuditManager):
@@ -32,10 +35,11 @@ class Operation_valueManager(AuditManager):
 
 class Operation_value(AuditModel):
     operation = ForeignKeyCascade(Operations)
-    edizm = ForeignKeyProtect(Ed_izm)
-    value = FloatField()
+    edizm = ForeignKeyProtect(Ed_izm, null=True, blank=True)
+    value = DecimalField(decimal_places=4, max_digits=19)
     props = BitField(flags=(
         ('perone', 'Из расчета на еденицу продукции'),
+        ('used_in_start', 'Учавствовало в учете запуска'),
         ('used_in_release', 'Учавствовало в учете выпуска'),
     ), default=0, db_index=True)
 
