@@ -2,7 +2,7 @@ import logging
 
 from bitfield import BitField
 from django.contrib.postgres.fields import JSONField
-from django.db.models import CheckConstraint, Q, F, UniqueConstraint, TextField, DecimalField, FloatField
+from django.db.models import CheckConstraint, Q, F, UniqueConstraint, TextField, DecimalField, PositiveIntegerField
 
 from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
 from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
@@ -24,7 +24,7 @@ class Launch_item_refsManager(AuditManager):
     @staticmethod
     def props():
         return BitField(flags=(
-            ('enabled', 'Доступность в данной производственной спецификации'),  # 1
+            ('used', 'Доступность в данной производственной спецификации'),  # 1
         ), default=1, db_index=True)
 
     @staticmethod
@@ -32,7 +32,7 @@ class Launch_item_refsManager(AuditManager):
         res = {
             'id': record.id,
             'props': record.props._value,
-            'enabled': record.props.enabled,
+            'used': record.props.used,
             'editing': record.editing,
             'deliting': record.deliting,
         }
@@ -44,14 +44,15 @@ class Launch_item_refsManager(AuditManager):
 
 class Launch_item_refs(AuditModel):
     child = ForeignKeyProtect(Item, related_name='launch_child')
+    item_full_name = TextField(db_index=True)
+    item_full_name_obj = JSONField(default=dict)
     item_refs = ForeignKeySetNull(Item_refs, blank=True, null=True)
     launch = ForeignKeyProtect(Launches)
     parent = ForeignKeyProtect(Item, related_name='launch_parent', blank=True, null=True)
     props = Launch_item_refsManager.props()
-    item_full_name = TextField()
-    item_full_name_obj = JSONField(default=dict)
-    qty = FloatField()
-    qty_per_one = FloatField()
+    qty = DecimalField(decimal_places=4, max_digits=19, db_index=True)
+    qty_per_one = DecimalField(decimal_places=4, max_digits=19, db_index=True)
+    replication_factor = PositiveIntegerField(default=1)
 
     objects = Launch_item_refsManager()
     tree_objects = TreeAuditModelManager()
@@ -73,6 +74,6 @@ class Launch_item_refs(AuditModel):
         verbose_name = 'Древо производственной спецификации'
         constraints = [
             CheckConstraint(check=~Q(child=F('parent')), name='Launch_item_refs_not_circulate_refs'),
-            UniqueConstraint(fields=['child', 'launch'], condition=Q(parent=None), name='Launch_item_refs_unique_constraint_0'),
-            UniqueConstraint(fields=['child', 'launch', 'parent'], name='Launch_item_refs_unique_constraint_1'),
+            UniqueConstraint(fields=['child_id', 'launch_id', 'props', 'qty', 'qty_per_one'], condition=Q(parent_id=None), name='Launch_item_refs_unique_constraint_0'),
+            UniqueConstraint(fields=['child_id', 'launch_id', 'parent_id', 'props', 'qty', 'qty_per_one'], name='Launch_item_refs_unique_constraint_1'),
         ]
