@@ -3,12 +3,12 @@ import logging
 from bitfield import BitField
 from django.conf import settings
 from django.db import transaction, connection
-from django.db.models import PositiveIntegerField, OneToOneField, PROTECT, CASCADE, UniqueConstraint, Q
+from django.db.models import PositiveIntegerField, OneToOneField, CASCADE, UniqueConstraint, Q
 from django.forms import model_to_dict
 
 from isc_common import delAttr, setAttr
 from isc_common.bit import IsBitOn
-from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade, ForeignKeySetNull
+from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
 from isc_common.http.DSRequest import DSRequest
 from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
 from isc_common.models.audit import AuditModel
@@ -51,7 +51,8 @@ class Launch_operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
 
     @staticmethod
     def refresh_resource(apps=None, schema_editor=None):
-        settings.LOCKS.acquire('Launch_operation_resourcesManager.refresh_resource')
+        key = 'Launch_operation_resourcesManager.refresh_resource'
+        settings.LOCKS.acquire(key)
         sql_text = '''select  launch_operationitem_id, count(*)
                 from production_launch_operation_resources
                 group by launch_operationitem_id
@@ -80,26 +81,31 @@ class Launch_operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
                             # print(res)
                         else:
                             first_step = False
-        settings.LOCKS.release('Launch_operation_resourcesManager.refresh_resource')
+        settings.LOCKS.release(key)
 
     def createFromRequest(self, request):
+        from kaf_pas.production.models.launch_item_view import Launch_item_viewManager
+
         request = DSRequest(request=request)
         data = request.get_data()
         _data = data.copy()
-        operationitem_id = _data.get('operationitem_id')
+        launch_operationitem_id = _data.get('launch_operationitem_id')
 
         res = []
-        if isinstance(operationitem_id, list):
-            delAttr(_data, 'operationitem_id')
+        if isinstance(launch_operationitem_id, list):
+            delAttr(_data, 'launch_operationitem_id')
             with transaction.atomic():
-                for operationitem in operationitem_id:
-                    setAttr(_data, 'operationitem_id', operationitem)
+                for launch_operationitem in launch_operationitem_id:
+                    setAttr(_data, 'launch_operationitem_id', launch_operationitem)
+
+                    Launch_item_viewManager.check_launch(Launch_operation_resources.objects.get(launch_operationitem_id=launch_operationitem).launch_operationitem.launch)
                     operation_resource = super().create(**_data)
                     _res = model_to_dict(operation_resource)
                     setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                     setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                     res.append(DelProps(_res))
         else:
+            Launch_item_viewManager.check_launch(Launch_operation_resources.objects.get(launch_operationitem_id=launch_operationitem_id).launch_operationitem.launch)
             operation_resource = super().create(**_data)
             _res = model_to_dict(operation_resource)
             setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
@@ -107,6 +113,22 @@ class Launch_operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
             res.append(DelProps(_res))
         return res
 
+    def deleteFromRequest(self, request, removed=None, ):
+        from kaf_pas.production.models.launch_item_view import Launch_item_viewManager
+
+        request = DSRequest(request=request)
+        res = 0
+        tuple_ids = request.get_tuple_ids()
+        with transaction.atomic():
+            for id, mode in tuple_ids:
+                if mode == 'hide':
+                    super().filter(id=id).soft_delete()
+                else:
+                    Launch_item_viewManager.check_launch(Launch_operation_resources.objects.get(id=id).launch_operationitem.launch)
+                    qty, _ = super().filter(id=id).delete()
+                res += qty
+        return res
+
     @staticmethod
     def getRecord(record):
         res = {
@@ -120,6 +142,10 @@ class Launch_operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
             'location__code': record.location.code if record.location else None,
             'location__name': record.location.name if record.location else None,
             'location__full_name': record.location.full_name if record.location else None,
+            'location_fin_id': record.location_fin.id if record.location_fin else None,
+            'location_fin__code': record.location_fin.code if record.location_fin else None,
+            'location_fin__name': record.location_fin.name if record.location_fin else None,
+            'location_fin__full_name': record.location_fin.full_name if record.location_fin else None,
             'complex_name': record.complex_name,
             'batch_size': record.batch_size,
             'props': record.props._value,
@@ -137,7 +163,8 @@ class Launch_operation_resources(AuditModel):
     operation_resources = ForeignKeySetNull(Operation_resources, null=True, blank=True)
     launch_operationitem = OneToOneField(Launch_operations_item, on_delete=CASCADE)
     resource = ForeignKeyProtect(Resource, null=True, blank=True)
-    location = ForeignKeyProtect(Locations, null=True, blank=True)
+    location = ForeignKeyProtect(Locations, related_name='Launch_operation_resources_location')
+    location_fin = ForeignKeyProtect(Locations, related_name='Launch_operation_resources_location_fin', null=True, blank=True)
     batch_size = PositiveIntegerField(default=1, db_index=True)
 
     props = BitField(flags=(
@@ -156,8 +183,12 @@ class Launch_operation_resources(AuditModel):
     class Meta:
         verbose_name = 'Кросс таблица'
         constraints = [
-            UniqueConstraint(fields=['launch_operationitem'], condition=Q(location=None) & Q(resource=None), name='Launch_operation_resources_unique_constraint_0'),
-            UniqueConstraint(fields=['launch_operationitem', 'resource'], condition=Q(location=None), name='Launch_operation_resources_unique_constraint_1'),
-            UniqueConstraint(fields=['launch_operationitem', 'location'], condition=Q(resource=None), name='Launch_operation_resources_unique_constraint_2'),
-            UniqueConstraint(fields=['launch_operationitem', 'location', 'resource'], name='Launch_operation_resources_unique_constraint_3'),
+            UniqueConstraint(fields=['launch_operationitem'], condition=Q(location=None) & Q(location_fin=None) & Q(resource=None), name='Launch_operation_resources_unique_constraint_0'),
+            UniqueConstraint(fields=['launch_operationitem', 'resource'], condition=Q(location=None) & Q(location_fin=None), name='Launch_operation_resources_unique_constraint_1'),
+            UniqueConstraint(fields=['launch_operationitem', 'location'], condition=Q(location_fin=None) & Q(resource=None), name='Launch_operation_resources_unique_constraint_2'),
+            UniqueConstraint(fields=['launch_operationitem', 'location', 'resource'], condition=Q(location_fin=None), name='Launch_operation_resources_unique_constraint_3'),
+            UniqueConstraint(fields=['launch_operationitem', 'location_fin'], condition=Q(location=None) & Q(resource=None), name='Launch_operation_resources_unique_constraint_4'),
+            UniqueConstraint(fields=['launch_operationitem', 'location_fin', 'resource'], condition=Q(location=None), name='Launch_operation_resources_unique_constraint_5'),
+            UniqueConstraint(fields=['launch_operationitem', 'location', 'location_fin'], condition=Q(resource=None), name='Launch_operation_resources_unique_constraint_6'),
+            UniqueConstraint(fields=['launch_operationitem', 'location', 'location_fin', 'resource'], name='Launch_operation_resources_unique_constraint_7'),
         ]
