@@ -1,12 +1,10 @@
 import logging
 
-from bitfield import BitField
 from django.db import transaction
 from django.db.models import PositiveIntegerField
 from django.forms import model_to_dict
 
 from isc_common import delAttr, setAttr
-from isc_common.bit import IsBitOn
 from isc_common.fields.description_field import DescriptionField
 from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
 from isc_common.http.DSRequest import DSRequest
@@ -15,6 +13,7 @@ from isc_common.models.base_ref import Hierarcy
 from isc_common.number import DelProps
 from kaf_pas.ckk.models.ed_izm import Ed_izm
 from kaf_pas.ckk.models.item import Item
+from kaf_pas.production.models.launch_item_view import Launch_item_viewManager
 from kaf_pas.production.models.launches import Launches
 from kaf_pas.production.models.operations import Operations
 from kaf_pas.production.models.operations_item import Operations_item
@@ -49,8 +48,6 @@ class Launch_operations_itemManager(CommonManagetWithLookUpFieldsManager):
             'description': record.description,
             'item_id': record.item.id,
             'parent_id': record.parent.id if record.parent else None,
-            'props': record.props._value,
-            'enabled': IsBitOn(record.props, 0),
             'editing': record.editing,
             'deliting': record.deliting,
         }
@@ -68,11 +65,14 @@ class Launch_operations_itemManager(CommonManagetWithLookUpFieldsManager):
                 launch_operationitem=launch_operationitem,
                 resource=operation_def_resource.resource,
                 location=operation_def_resource.location,
+                location_fin=operation_def_resource.location_fin,
             )
 
     def createFromRequest(self, request):
         request = DSRequest(request=request)
         data = request.get_data()
+
+        Launch_item_viewManager.check_launch(launch=Launches.objects.get(id=data.get('launch_id')))
         _data = data.copy()
         delAttr(_data, 'operation__full_name')
 
@@ -109,6 +109,7 @@ class Launch_operations_itemManager(CommonManagetWithLookUpFieldsManager):
     def updateFromRequest(self, request, removed=None, function=None):
         request = DSRequest(request=request)
         data = request.get_data()
+        Launch_item_viewManager.check_launch(launch=Launches.objects.get(id=data.get('launch_id')))
         _data = data.copy()
 
         delAttr(_data, 'operation__full_name')
@@ -118,19 +119,30 @@ class Launch_operations_itemManager(CommonManagetWithLookUpFieldsManager):
 
         return data
 
+    def deleteFromRequest(self, request, removed=None, ):
+        request = DSRequest(request=request)
+        res = 0
+        tuple_ids = request.get_tuple_ids()
+        with transaction.atomic():
+            for id, mode in tuple_ids:
+                if mode == 'hide':
+                    super().filter(id=id).soft_delete()
+                else:
+                    Launch_item_viewManager.check_launch(launch=Launch_operations_item.objects.get(id=id).launch)
+                    qty, _ = super().filter(id=id).delete()
+                res += qty
+        return res
+
 
 class Launch_operations_item(Hierarcy):
     description = DescriptionField()
     ed_izm = ForeignKeyProtect(Ed_izm, null=True, blank=True)
     item = ForeignKeyProtect(Item)
     launch = ForeignKeyProtect(Launches)
-    num = PositiveIntegerField(null=True, blank=True, db_index=True)
+    num = PositiveIntegerField(db_index=True, default=None)
     operation = ForeignKeyProtect(Operations)
     operationitem = ForeignKeySetNull(Operations_item, null=True, blank=True)
     qty = PositiveIntegerField(null=True, blank=True, db_index=True)
-    props = BitField(flags=(
-        ('enabled', 'Доступность в данной производственной спецификации'),  # 1
-    ), default=1, db_index=True)
 
     objects = Launch_operations_itemManager()
 
@@ -142,4 +154,4 @@ class Launch_operations_item(Hierarcy):
 
     class Meta:
         verbose_name = 'Кросс таблица'
-        unique_together = (('item', 'launch', 'operation'),)
+        unique_together = (('item', 'launch', 'operation', 'num'),)
