@@ -29,6 +29,7 @@ class Operation_resourcesQuerySet(CommonManagetWithLookUpFieldsQuerySet):
 
         if not kwargs.get('resource_id') and not kwargs.get('location_id'):
             raise Exception('Необходим хотябы один выбранный параметр.')
+
         delAttr(kwargs, 'resource__name')
         delAttr(kwargs, 'location__full_name')
 
@@ -47,7 +48,8 @@ class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
 
     @staticmethod
     def refresh_resource(apps=None, schema_editor=None):
-        settings.LOCKS.acquire('Operation_resourcesManager.refresh_resource')
+        key='Operation_resourcesManager.refresh_resource'
+        settings.LOCKS.acquire(key)
         sql_text = '''select  operationitem_id, count(*)
                     from production_operation_resources
                     group by operationitem_id
@@ -76,7 +78,7 @@ class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
                             # print(res)
                         else:
                             first_step = False
-        settings.LOCKS.release('Operation_resourcesManager.refresh_resource')
+        settings.LOCKS.release(key)
 
     def createFromRequest(self, request):
         request = DSRequest(request=request)
@@ -96,7 +98,7 @@ class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
                     if created:
                         _res = model_to_dict(operation_resource)
                         setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
-                        setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
+                        setAttr(_res, 'resource__name', operation_resource.resource.name if operation_resource.resource else None)
                         res.append(_res)
         else:
             delAttr(_data, 'location__full_name')
@@ -132,6 +134,10 @@ class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
             'location__code': record.location.code if record.location else None,
             'location__name': record.location.name if record.location else None,
             'location__full_name': record.location.full_name if record.location else None,
+            'location_fin_id': record.location_fin.id if record.location_fin else None,
+            'location_fin__code': record.location_fin.code if record.location_fin else None,
+            'location_fin__name': record.location_fin.name if record.location_fin else None,
+            'location_fin__full_name': record.location_fin.full_name if record.location_fin else None,
             'complex_name': record.complex_name,
             'batch_size': record.batch_size,
             'editing': record.editing,
@@ -146,7 +152,8 @@ class Operation_resourcesManager(CommonManagetWithLookUpFieldsManager):
 class Operation_resources(AuditModel):
     operationitem = OneToOneField(Operations_item, on_delete=CASCADE)
     resource = ForeignKeyProtect(Resource, null=True, blank=True)
-    location = ForeignKeyProtect(Locations, null=True, blank=True)
+    location = ForeignKeyProtect(Locations, null=True, blank=True, related_name='Operation_resources_location')
+    location_fin = ForeignKeyProtect(Locations, null=True, blank=True, related_name='Operation_resources_location_fin')
     batch_size = PositiveIntegerField(default=1)
 
     @property
@@ -164,9 +171,14 @@ class Operation_resources(AuditModel):
     class Meta:
         verbose_name = 'Кросс таблица'
         constraints = [
-            UniqueConstraint(fields=['operationitem'], condition=Q(location=None) & Q(resource=None), name='Operation_resources_unique_constraint_0'),
-            UniqueConstraint(fields=['operationitem', 'resource'], condition=Q(location=None), name='Operation_resources_unique_constraint_1'),
-            UniqueConstraint(fields=['location', 'operationitem'], condition=Q(resource=None), name='Operation_resources_unique_constraint_2'),
-            UniqueConstraint(fields=['location', 'operationitem', 'resource'], name='Operation_resources_unique_constraint_3'),
+            UniqueConstraint(fields=['operationitem'], condition=Q(location=None) & Q(location_fin=None) & Q(resource=None), name='Operation_resources_unique_constraint_0'),
+            UniqueConstraint(fields=['operationitem', 'resource'], condition=Q(location=None) & Q(location_fin=None), name='Operation_resources_unique_constraint_1'),
+            UniqueConstraint(fields=['location', 'operationitem'], condition=Q(location_fin=None) & Q(resource=None), name='Operation_resources_unique_constraint_2'),
+            UniqueConstraint(fields=['location', 'operationitem', 'resource'], condition=Q(location_fin=None), name='Operation_resources_unique_constraint_3'),
+            UniqueConstraint(fields=['location_fin', 'operationitem'], condition=Q(location=None) & Q(resource=None), name='Operation_resources_unique_constraint_4'),
+            UniqueConstraint(fields=['location_fin', 'operationitem', 'resource'], condition=Q(location=None), name='Operation_resources_unique_constraint_5'),
+            UniqueConstraint(fields=['location', 'location_fin', 'operationitem'], condition=Q(resource=None), name='Operation_resources_unique_constraint_6'),
+            UniqueConstraint(fields=['location', 'location_fin', 'operationitem', 'resource'], name='Operation_resources_unique_constraint_7'),
+
         ]
 
