@@ -27,7 +27,8 @@ class Operations_itemManager(CommonManagetWithLookUpFieldsManager):
 
     @staticmethod
     def refresh_num1():
-        settings.LOCKS.acquire('Operations_itemManager.refresh_num1')
+        key='Operations_itemManager.refresh_num1'
+        settings.LOCKS.acquire(key)
         sql = '''select item_id,
                            count(*) 
                     from production_operations_item
@@ -58,12 +59,13 @@ class Operations_itemManager(CommonManagetWithLookUpFieldsManager):
                                 operations_item.save()
                                 cnt += 1
             print(f'Corected: {cnt} operations.')
-        settings.LOCKS.release('Operations_itemManager.refresh_num1')
+        settings.LOCKS.release(key)
         return cnt
 
     @staticmethod
     def refresh_num(apps, schema_editor):
-        settings.LOCKS.acquire('Operations_itemManager.refresh_num')
+        key='Operations_itemManager.refresh_num'
+        settings.LOCKS.acquire(key)
         sql = '''select num,
                        item_id,
                        count(*) 
@@ -92,7 +94,7 @@ class Operations_itemManager(CommonManagetWithLookUpFieldsManager):
                         num_max += 1
                         operations_item.num = num_max
                         operations_item.save()
-        settings.LOCKS.release('Operations_itemManager.refresh_num')
+        settings.LOCKS.release(key)
 
     @staticmethod
     def getRecord(record):
@@ -122,6 +124,9 @@ class Operations_itemManager(CommonManagetWithLookUpFieldsManager):
     def _rec_def_resources(self, operationitem):
         from kaf_pas.production.models.operation_def_resources import Operation_def_resources
         from kaf_pas.production.models.operation_resources import Operation_resources
+        query = Operation_def_resources.objects.filter(operation=operationitem.operation)
+        if query.count() > 1:
+            raise Exception(f'К данному типу операции по-умолчанию привязано более одного ресурса или места выполнения.')
         for operation_def_resource in Operation_def_resources.objects.filter(operation=operationitem.operation):
             Operation_resources.objects.get_or_create(
                 operationitem=operationitem,
@@ -194,11 +199,11 @@ class Operations_itemManager(CommonManagetWithLookUpFieldsManager):
 
 class Operations_item(Hierarcy):
     description = DescriptionField()
-    item = ForeignKeyProtect(Item)
-    operation = ForeignKeyProtect(Operations)
-    num = PositiveIntegerField(default=None, null=True, blank=True)
-    qty = PositiveIntegerField(default=None, null=True, blank=True)
     ed_izm = ForeignKeyProtect(Ed_izm, default=None, null=True, blank=True)
+    item = ForeignKeyProtect(Item)
+    num = PositiveIntegerField(default=None, null=True, blank=True)
+    operation = ForeignKeyProtect(Operations)
+    qty = PositiveIntegerField(default=None, null=True, blank=True)
 
     objects = Operations_itemManager()
 
